/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef BALLBARPATTERNHEADERFILE
#define BALLBARPATTERNHEADERFILE

#include <QString>
#include <QList>
#include <QUrl>
#include <vector>

#include "../../core/vector2.h"
#include "../../core/vector3.h"

class TBallBarPattern
{
public:

struct TBallBarMeasurement
{
	QString								name;
	double								length;
	double								sphere_form1;
	double								sphere_form2;
	TVector3							sphere_position1;
	TVector3							sphere_position2;
	TVector3							probe_offset;
};

// CREATORS
	TBallBarPattern(void);
	~TBallBarPattern(void);

// ACCESSORS
	QString Last_Error(void) const {return d_last_error;}

	double Length_Min(void) const {return d_pattern_min_length;}
	double Length_Max(void) const {return d_pattern_max_length;}
	double Length_Average(void) const {return d_pattern_average;}
	double Length_Stdev(void) const {return d_pattern_stdev;}
	
	TVector3 Volume_Corner_Min(void) const;
	TVector3 Volume_Corner_Max(void) const;

	int Measurement_Count(void) const {return static_cast<int>(d_ballbar_measurements.size());}
	TBallBarMeasurement BallBar_Measurement(const int index) const;
	
	std::vector<TBallBarMeasurement> BallBar_Pattern(void) const {return d_ballbar_measurements;}

// MANIPULATORS
	bool Load_Data(const QString &file_name);
	void Clear(void);

private:
	std::vector<TBallBarMeasurement>	d_ballbar_measurements;
	double								d_pattern_average;
	double								d_pattern_stdev;
	double								d_pattern_min_length;
	double								d_pattern_max_length;
	
	mutable QString						d_last_error;

enum TFileEncoding
{
	ASCII_FILE_TYPE = 0,
	BINARY_FILE_TYPE
};

enum TFileContents
{
	UNKNOWN_CONTENTS = 0,
	BALLBAR_DATAFILE_LEGACY_V1,
	BALLBAR_DATAFILE_LEGACY_V2,
	BALLBAR_DATAFILE_V1
};

	TFileEncoding DetermineFileEncoding(const QByteArray &file_data) const;
	TFileContents DetermineAsciiFileContents(const QByteArray &file_data) const;
	
	bool Load_Data_Legacy_V1(const QByteArray &file_data);
	bool Load_Data_Legacy_V2(const QByteArray &file_data);
	bool Load_Data_V1(const QByteArray &file_data);
	
	void Update_Statistics(void);

// NOT IMPLEMENTED
	TBallBarPattern(const TBallBarPattern&);
	TBallBarPattern& operator=(const TBallBarPattern&);
};

#endif
