/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////



#ifndef BALLBARVIEWERHEADERFILE
#define BALLBARVIEWERHEADERFILE

#include <QMainWindow>

#include "ballbarpattern.h"

class QAction;
class QSettings;
class QSplitter;
class QPlainTextEdit;
class QCloseEvent;
class QDragEnterEvent;
class QDropEvent;

class TGraphWidget;
class TToleranceWidget;
class TMeasurementWidget;
class TOpenGLDisplayWidget;
class TMessageBox;

class TBallBarViewer : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TBallBarViewer(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TBallBarViewer(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	
	void File_New(void);
	void File_Import_Ballbar(void);
	void Menu_About(void);
	
	void Tolerance_Changed(void);
	void Ballbar_Item_Clicked(const int);

protected:
	virtual void dragEnterEvent(QDragEnterEvent *DragEnterEvent);
	virtual void dropEvent(QDropEvent *DropEvent);
	virtual void closeEvent(QCloseEvent *event);

private:
	QAction								*d_file_new_action;
	QAction								*d_file_import_ballbar_action;
	QAction								*d_file_quit_action;
	QAction								*d_help_about_action;
	TToleranceWidget					*d_tolerance_widget;
	TMeasurementWidget					*d_measurement_widget;
	TOpenGLDisplayWidget				*d_opengl_display_widget;
	QSplitter							*d_splitter_widget;
	TMessageBox							*d_msg_box;
	QPlainTextEdit						*d_analysis_text_edit;
	TGraphWidget 						*d_analysis_graph_widget;
	QSettings							*d_settings;
	
	TBallBarPattern						d_ballbar_pattern;
	QString								d_application_path;
	QString								d_data_path;
	
	void Import_Ballbar_Data(const QString &file_name);
	void Update_Display(const bool scale_to_fit);


// NOT IMPLEMENTED
	TBallBarViewer(const TBallBarViewer&);
	TBallBarViewer& operator=(const TBallBarViewer&);
};

#endif
