/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpacerItem>

#include "measurementwidget.h"

TMeasurementWidget::TMeasurementWidget(
	const QWidget						*parent,
	Qt::WindowFlags 					flags)
:QWidget(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*widget_layout;
	QFrame								*separator_hline0;
	QFrame								*separator_hline1;
	QFrame								*separator_hline2;
	QFrame								*separator_hline3;
	QFrame								*separator_hline4;
	QLabel								*deviation_mean_label;
	QLabel								*deviation_median_label;
	QLabel								*length_label;
	QLabel								*position_label;
	QLabel								*probe_x_label;
	QLabel								*probe_y_label;
	QLabel								*probe_z_label;
	QLabel								*sph1_z_label;
	QLabel								*sph1_x_label;
	QLabel								*sph1_form_label;
	QLabel								*sph1_y_label;
	QLabel								*sph2_y_label;
	QLabel								*sph2_form_label;
	QLabel								*sph2_x_label;
	QLabel								*sph2_z_label;
	QLabel								*units_label;
	QSpacerItem							*widget_vspacer;
	QSizePolicy							sizepolicy_minimumexpanding_fixed(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
	
	sizepolicy_minimumexpanding_fixed.setHorizontalStretch(0);
	sizepolicy_minimumexpanding_fixed.setVerticalStretch(0);
	
	widget_layout = new QGridLayout(this);
	widget_layout->setSpacing(4);
	widget_layout->setContentsMargins(4,4,4,4);

	position_label = new QLabel(this);
	position_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(position_label,0,0,1,1);
	
	d_position_edit = new QLineEdit(this);
	d_position_edit->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
	d_position_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_position_edit->sizePolicy().hasHeightForWidth());
	d_position_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_position_edit,0,1,1,1);
	
	separator_hline0 = new QFrame(this);
	separator_hline0->setFrameShape(QFrame::HLine);
	separator_hline0->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline0,1,0,1,2);

	length_label = new QLabel(this);
	length_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(length_label,2,0,1,1);
	
	d_length_edit = new QLineEdit(this);
	d_length_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_length_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_length_edit->sizePolicy().hasHeightForWidth());
	d_length_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_length_edit,2,1,1,1);

	deviation_mean_label = new QLabel(this);
	deviation_mean_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(deviation_mean_label,3,0,1,1);
	
	d_deviation_mean_edit = new QLineEdit(this);
	d_deviation_mean_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_deviation_mean_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_deviation_mean_edit->sizePolicy().hasHeightForWidth());
	d_deviation_mean_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_deviation_mean_edit,3,1,1,1);

	deviation_median_label = new QLabel(this);
	deviation_median_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(deviation_median_label,4,0,1,1);
	
	d_deviation_median_edit = new QLineEdit(this);
	d_deviation_median_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_deviation_median_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_deviation_median_edit->sizePolicy().hasHeightForWidth());
	d_deviation_median_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_deviation_median_edit,4,1,1,1);

	separator_hline1 = new QFrame(this);
	separator_hline1->setFrameShape(QFrame::HLine);
	separator_hline1->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline1,5,0,1,2);

	sph1_x_label = new QLabel(this);
	sph1_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph1_x_label,6,0,1,1);
	
	d_sph1_x_edit = new QLineEdit(this);
	d_sph1_x_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph1_x_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph1_x_edit->sizePolicy().hasHeightForWidth());
	d_sph1_x_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph1_x_edit,6,1,1,1);

	sph1_y_label = new QLabel(this);
	sph1_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph1_y_label,7,0,1,1);
	
	d_sph1_y_edit = new QLineEdit(this);
	d_sph1_y_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph1_y_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph1_y_edit->sizePolicy().hasHeightForWidth());
	d_sph1_y_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph1_y_edit,7,1,1,1);

	sph1_z_label = new QLabel(this);
	sph1_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph1_z_label,8,0,1,1);
	
	d_sph1_z_edit = new QLineEdit(this);
	d_sph1_z_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph1_z_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph1_z_edit->sizePolicy().hasHeightForWidth());
	d_sph1_z_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph1_z_edit,8,1,1,1);
	
	sph1_form_label = new QLabel(this);
	sph1_form_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph1_form_label,9,0,1,1);
	
	d_sph1_form_edit = new QLineEdit(this);
	d_sph1_form_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph1_form_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph1_form_edit->sizePolicy().hasHeightForWidth());
	d_sph1_form_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph1_form_edit,9,1,1,1);
	
	separator_hline2 = new QFrame(this);
	separator_hline2->setFrameShape(QFrame::HLine);
	separator_hline2->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline2,10,0,1,2);

	sph2_x_label = new QLabel(this);
	sph2_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph2_x_label,11,0,1,1);

	d_sph2_x_edit = new QLineEdit(this);
	d_sph2_x_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph2_x_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph2_x_edit->sizePolicy().hasHeightForWidth());
	d_sph2_x_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph2_x_edit,11,1,1,1);
	
	sph2_y_label = new QLabel(this);
	sph2_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph2_y_label,12,0,1,1);
	
	d_sph2_y_edit = new QLineEdit(this);
	d_sph2_y_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph2_y_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph2_y_edit->sizePolicy().hasHeightForWidth());
	d_sph2_y_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph2_y_edit,12,1,1,1);

	sph2_z_label = new QLabel(this);
	sph2_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph2_z_label,13,0,1,1);
	
	d_sph2_z_edit = new QLineEdit(this);
	d_sph2_z_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph2_z_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph2_z_edit->sizePolicy().hasHeightForWidth());
	d_sph2_z_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph2_z_edit,13,1,1,1);
	
	sph2_form_label = new QLabel(this);
	sph2_form_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(sph2_form_label,14,0,1,1);
	
	d_sph2_form_edit = new QLineEdit(this);
	d_sph2_form_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_sph2_form_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_sph2_form_edit->sizePolicy().hasHeightForWidth());
	d_sph2_form_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_sph2_form_edit,14,1,1,1);
	
	separator_hline3 = new QFrame(this);
	separator_hline3->setFrameShape(QFrame::HLine);
	separator_hline3->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline3,15,0,1,2);

	probe_x_label = new QLabel(this);
	probe_x_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_x_label,16,0,1,1);
	
	d_probe_x_edit = new QLineEdit(this);
	d_probe_x_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_probe_x_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_probe_x_edit->sizePolicy().hasHeightForWidth());
	d_probe_x_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_probe_x_edit,16,1,1,1);
	
	probe_y_label = new QLabel(this);
	probe_y_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_y_label,17,0,1,1);
	
	d_probe_y_edit = new QLineEdit(this);
	d_probe_y_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_probe_y_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_probe_y_edit->sizePolicy().hasHeightForWidth());
	d_probe_y_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_probe_y_edit,17,1,1,1);
	
	probe_z_label = new QLabel(this);
	probe_z_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(probe_z_label,18,0,1,1);
	
	d_probe_z_edit = new QLineEdit(this);
	d_probe_z_edit->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	d_probe_z_edit->setReadOnly(true);
	sizepolicy_minimumexpanding_fixed.setHeightForWidth(d_probe_z_edit->sizePolicy().hasHeightForWidth());
	d_probe_z_edit->setSizePolicy(sizepolicy_minimumexpanding_fixed);
	widget_layout->addWidget(d_probe_z_edit,18,1,1,1);
	
	separator_hline4 = new QFrame(this);
	separator_hline4->setFrameShape(QFrame::HLine);
	separator_hline4->setFrameShadow(QFrame::Sunken);
	widget_layout->addWidget(separator_hline4,19,0,1,2);

	units_label = new QLabel(this);
	units_label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
	widget_layout->addWidget(units_label,20,0,1,2);
	
	widget_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	widget_layout->addItem(widget_vspacer,21,0,1,2);

	QWidget::setTabOrder(d_position_edit,d_length_edit);
	QWidget::setTabOrder(d_length_edit,d_deviation_mean_edit);
	QWidget::setTabOrder(d_deviation_mean_edit,d_deviation_median_edit);
	QWidget::setTabOrder(d_deviation_median_edit,d_sph1_x_edit);
	QWidget::setTabOrder(d_sph1_x_edit,d_sph1_y_edit);
	QWidget::setTabOrder(d_sph1_y_edit,d_sph1_z_edit);
	QWidget::setTabOrder(d_sph1_z_edit,d_sph1_form_edit);
	QWidget::setTabOrder(d_sph1_form_edit,d_sph2_x_edit);
	QWidget::setTabOrder(d_sph2_x_edit,d_sph2_y_edit);
	QWidget::setTabOrder(d_sph2_y_edit,d_sph2_z_edit);
	QWidget::setTabOrder(d_sph2_z_edit,d_sph2_form_edit);
	QWidget::setTabOrder(d_sph2_form_edit,d_probe_x_edit);
	QWidget::setTabOrder(d_probe_x_edit,d_probe_y_edit);
	QWidget::setTabOrder(d_probe_y_edit,d_probe_z_edit);
		
	position_label->setText(QStringLiteral("Selected Position:"));
	length_label->setText(QStringLiteral("Length:"));
	deviation_mean_label->setText(QStringLiteral("Dev. From Mean:"));
	deviation_median_label->setText(QStringLiteral("Dev. From Median:"));
	sph1_x_label->setText(QStringLiteral("Sphere 1 X:"));
	sph1_y_label->setText(QStringLiteral("Sphere 1 Y:"));
	sph1_z_label->setText(QStringLiteral("Sphere 1 Z:"));
	sph1_form_label->setText(QStringLiteral("Sphere 1 Form:"));
	sph2_x_label->setText(QStringLiteral("Sphere 2 X:"));
	sph2_y_label->setText(QStringLiteral("Sphere 2 Y:"));
	sph2_z_label->setText(QStringLiteral("Sphere 2 Z:"));
	sph2_form_label->setText(QStringLiteral("Sphere 2 Form:"));
	probe_x_label->setText(QStringLiteral("Probe Offset X:"));
	probe_y_label->setText(QStringLiteral("Probe Offset Y:"));
	probe_z_label->setText(QStringLiteral("Probe Offset Z:"));
	units_label->setText(QStringLiteral("Reported values in mm."));

	// defaults
	this->setEnabled(false);
}

TMeasurementWidget::~TMeasurementWidget(void)
{
}

void TMeasurementWidget::Enable_Display(
	const bool							state)
{
	this->setEnabled(state);
	
	if(!state)
	{
		d_position_edit->setText(QString());
		d_length_edit->setText(QString());
		d_deviation_mean_edit->setText(QString());
		d_deviation_median_edit->setText(QString());
		d_sph1_x_edit->setText(QString());
		d_sph1_y_edit->setText(QString());
		d_sph1_z_edit->setText(QString());
		d_sph1_form_edit->setText(QString());
		d_sph2_x_edit->setText(QString());
		d_sph2_y_edit->setText(QString());
		d_sph2_z_edit->setText(QString());
		d_sph2_form_edit->setText(QString());
		d_probe_x_edit->setText(QString());
		d_probe_y_edit->setText(QString());
		d_probe_z_edit->setText(QString());
	}
}

void TMeasurementWidget::Set_Position_Name(
	const QString						&name)
{
	d_position_edit->setText(name);
}

void TMeasurementWidget::Set_Length(
	const double						&length)
{
	d_length_edit->setText(QString("%1").arg(length,0,'f',4));
}

void TMeasurementWidget::Set_Mean_Deviation(
	const double						&deviation)
{
	d_deviation_mean_edit->setText(QString("%1").arg(deviation,0,'f',4));
}

void TMeasurementWidget::Set_Median_Deviation(
	const double						&deviation)
{
	d_deviation_median_edit->setText(QString("%1").arg(deviation,0,'f',4));
}

void TMeasurementWidget::Set_Sphere1(
	const TVector3						&pos,
	const double						&form)
{
	d_sph1_x_edit->setText(QString("%1").arg(pos.x,0,'f',4));
	d_sph1_y_edit->setText(QString("%1").arg(pos.y,0,'f',4));
	d_sph1_z_edit->setText(QString("%1").arg(pos.z,0,'f',4));
	d_sph1_form_edit->setText(QString("%1").arg(form,0,'f',4));
}

void TMeasurementWidget::Set_Sphere2(
	const TVector3						&pos,
	const double						&form)
{
	d_sph2_x_edit->setText(QString("%1").arg(pos.x,0,'f',4));
	d_sph2_y_edit->setText(QString("%1").arg(pos.y,0,'f',4));
	d_sph2_z_edit->setText(QString("%1").arg(pos.z,0,'f',4));
	d_sph2_form_edit->setText(QString("%1").arg(form,0,'f',4));
}

void TMeasurementWidget::Set_Probe(
	const TVector3						&vec)
{
	d_probe_x_edit->setText(QString("%1").arg(vec.x,0,'f',4));
	d_probe_y_edit->setText(QString("%1").arg(vec.y,0,'f',4));
	d_probe_z_edit->setText(QString("%1").arg(vec.z,0,'f',4));
}
