/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef OPENGLDISPLAYHEADERFILE
#define OPENGLDISPLAYHEADERFILE

#include <vector>

#include "../../core/openglwidget.h"
#include "../../core/openglvertexbuffer.h"
#include "../../core/openglgeometryfactory.h"

class QOpenGLShaderProgram;
class TOpenGLVertexBuffer;

class TOpenGLDisplay : public TOpenGLWidget
{
	Q_OBJECT
public:

struct TBallBarItem
{
	TVector3							Sphere1;
	TVector3							Sphere2;
};
	
struct TBallBar
{
	int									id;
	QString								name;
	bool								show_oot;
	bool								show_selected;
	bool								show_translucent;
	
	TVector3							min_xyz;
	TVector3							max_xyz;
	TVector3							name_position_space;
	TVector3							name_position_bar;

	int									ballbar_shaft_vertex_id;
	int									ballbar_metal_vertex_id;
	int									ballbar_shaft_normal_id;
	int									ballbar_metal_normal_id;

	int									probe_stem_vertex_id;
	int									probe_ruby_vertex_id;
	int									probe_stem_normal_id;
	int									probe_ruby_normal_id;

	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> ballbar_shaft;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> ballbar_spheres;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_stem;
	std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> probe_ruby;
};


// CREATORS
	TOpenGLDisplay(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TOpenGLDisplay(void);

// ACCESSORS
	int Ball_Bar_Count(void) const {return static_cast<int>(d_BallBarItems.size());}

// MANIPULATORS
	void Define_Machine_Volume(const TVector3 &pnt_min, const TVector3 &pnt_max);

	void Clear_Selection(void);
	void Set_Selection(const int id);
	
	void Clear_BallBars(void);
	void Add_BallBar(const TBallBar &ballbar);

protected:

	virtual void GLInit(void);
	virtual void GLResize(const int width, const int height);
	virtual void GLRender(void);

private:

	std::vector<TBallBarItem>			d_BallBarItems;
	
	TVector3							d_min_xyz;
	TVector3							d_max_xyz;
	TVector3							d_axis_draw_pos;
	double								d_axis_draw_len;
	std::vector<TBallBar>				d_ballbars;

	TOpenGLVertexBuffer					d_opengl_vertex_buffer;
	
	QOpenGLShaderProgram				*d_triangle_shader_program;
	QOpenGLShaderProgram				*d_basic_shader_program;

	GLint								d_triangle_uniform_color;
	GLint								d_triangle_uniform_specular;
	GLint								d_triangle_uniform_offset;
	GLint								d_triangle_attribute_vertex;
	GLint								d_triangle_attribute_normal;
	
	GLint								d_basic_uniform_color;
	GLint								d_basic_attribute_vertex;
	
	void Render_Triangle_Buffer(const unsigned int size,const TVector3 &offset,const TVector3 &color, const double &specular,const double &opacity, double * const vertex,double * const nominal);
	void Render_Line_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Line_Strip_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);
	void Render_Point_Buffer(const unsigned int size,const TVector3 &color, double * const vertex);

	unsigned int Convert_Geometry_To_Buffer(const std::vector<TOpenGLGeometryFactory::TOpenGLTriangleVertex> &data,double ** const vertex,double ** const normal) const;

// NOT IMPLEMENTED
	TOpenGLDisplay(const TOpenGLDisplay&);
	TOpenGLDisplay& operator=(const TOpenGLDisplay&);
};

#endif

