/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef OPENGLDISPLAYWIDGETHEADERFILE
#define OPENGLDISPLAYWIDGETHEADERFILE

#include <vector>
#include <QFrame>
#include <QColor>

#include "../../core/vector3.h"
#include "../../core/vector2.h"

class QToolButton;
class QLabel;
class TOpenGLDisplay;
class TMeasurement;

class TOpenGLDisplayWidget : public QFrame
{
Q_OBJECT

public:

// CREATORS
	TOpenGLDisplayWidget(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TOpenGLDisplayWidget(void);

// ACCESSORS

// MANIPULATORS
	void Define_Machine_Volume(const TVector3 &pnt_min, const TVector3 &pnt_max);
	
	void Add_BallBar(const int id,const QString &name,const TVector3 &sph1, const TVector3 &sph2, const TVector3 &probe,const bool oot, const bool translucent);
	
	void Clear_Selection(void);
	void Set_Selection(const int id);
	
	void Clear_BallBars(void);

	void Update(const bool scale_to_fit);

signals:
	void Ballbar_Item_Clicked(int);

private slots:
	void Model_Item_Clicked(const double&,const double&,const double&,const int);
	
	void Scale_To_Fit(void);
	void Rotate2d(bool);
	void Rotate3d(bool);
	void Pan(bool);

private:
	
	QToolButton							*d_pan_button;
	QToolButton							*d_rotate_2d_button;
	QToolButton							*d_rotate_3d_button;
	QToolButton							*d_scale_to_fit_button;
	QLabel								*d_gldisplay_error_label;
	TOpenGLDisplay						*d_opengl_display;
	
	TVector3							d_min_xyz;
	TVector3							d_max_xyz;

	

// NOT IMPLEMENTED
	TOpenGLDisplayWidget(const TOpenGLDisplayWidget&);
	TOpenGLDisplayWidget& operator=(const TOpenGLDisplayWidget&);
};

#endif
