/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QHBoxLayout>
#include <QPushButton>
#include <QSpacerItem>
#include <QTextBrowser>
#include <QVBoxLayout>

#include "tolerancehelpdialog.h"

TToleranceHelpDialog::TToleranceHelpDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QHBoxLayout							*button_hlayout;
	QPushButton							*close_button;
	QSpacerItem							*button_hspacer;
	QTextBrowser						*help_text;
	QVBoxLayout							*dialog_vlayout;


	this->resize(551,551);

	dialog_vlayout = new QVBoxLayout(this);

	help_text = new QTextBrowser(this);
	help_text->setHtml(QStringLiteral("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">" \
			"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">" \
			"p, li { white-space: pre-wrap; }" \
			"</style></head><body style=\" font-family:'Arial'; font-size:10pt; font-weight:400; font-style:normal;\">" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:12pt; font-weight:600; text-decoration: underline;\">Tolerance Expression Formats</span></p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The tolerance expression can be in one of the supported forms.  The supported forms are listed below:</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; text-decoration: underline;\">Fixed Value</span></p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Tolerances are entered, in metric, as a single value.  The tolerance is independent of the measurement length.</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Example:  0.010</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; text-decoration: underline;\">Length Dependent Expression</span></p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Tolerances are entered, in metric, as an expression that includes the length.  The expression consists of a constant and length dependent components.</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Example:  0.010 + 0.020L/1200</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; text-decoration: underline;\">Length Dependent Expression with Limit</span></p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Tolerances are entered, in metric, as an expression that includes the length and a maximum value.  The expression consists of a constant and length dependent components followed by the upper limit.</p><br>" \
			"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Example:  0.010 + 0.020L/1200 | 0.020</p><br>"));
	dialog_vlayout->addWidget(help_text);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	close_button = new QPushButton(this);
	button_hlayout->addWidget(close_button);
	dialog_vlayout->addLayout(button_hlayout);

	this->setWindowTitle(QStringLiteral("Tolerance Help"));
	close_button->setText(QStringLiteral("Close"));
	
	connect(close_button,&QPushButton::clicked,this,&TToleranceHelpDialog::close);
}

TToleranceHelpDialog::~TToleranceHelpDialog(void)
{
}

