/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2024  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef TOUCHPOINTDOCKHEADERFILE
#define TOUCHPOINTDOCKHEADERFILE

#include <QWidget>

class QLabel;
class QLineEdit;
class QToolButton;
class QSlider;
class TToleranceHelpDialog;

class TToleranceWidget : public QWidget
{
Q_OBJECT
	
public:
	
enum TToleranceType
{
	TOLERANCE_ERROR = 0,
	TOLERANCE_BAND,
	TOLERANCE_FORMULA,
	TOLERANCE_FORMULA_WITH_LIMIT
};


// CREATORS
	TToleranceWidget(const QWidget *parent=0,Qt::WindowFlags flags=Qt::WindowFlags());
	~TToleranceWidget(void);
	
// ACCESSORS
	TToleranceType Tolerance_Type(void) const {return d_tolerance_type;}
	double Tolerance_A(void) const {return d_tolerance_a;}
	double Tolerance_B(void) const {return d_tolerance_b;}
	double Tolerance_C(void) const {return d_tolerance_c;}
	double Tolerance_D(void) const {return d_tolerance_d;}
	
	double Tolerance_Value(const double &length) const;
	double Sub_Tolerance_Percent(void) const;
	
	double Tolerance_Limit_Position(const double &tol_a,const double &tol_b,const double &tol_c,const double &tol_d) const;	// Position where fixed tolerance takes over
	
	inline double Tolerance_Limit_Position(void) const
	{
		return this->Tolerance_Limit_Position(d_tolerance_a,d_tolerance_b,d_tolerance_c,d_tolerance_d);
	}
	
// MANIPULATORS
	void Set_Tolerance_Type(const TToleranceType &type) {d_tolerance_type = type;}
	void Set_Tolerance_A(const double &d) {d_tolerance_a = d;}
	void Set_Tolerance_B(const double &d) {d_tolerance_b = d;}
	void Set_Tolerance_C(const double &d) {d_tolerance_c = d;}
	void Set_Tolerance_D(const double &d) {d_tolerance_d = d;}
	
	void Update(void);					// update display of tolerance
	
	void Enable_Results(const bool state);
	void Set_Results(const double &range,const double &nominal_length);

signals:
	void Tolerance_Changed(void);
	void Sub_Tolerance_Changed(void);
	
private slots:
	void Update_Tolerance(void);
	void Apply_Tolerance(void);
	void Change_Sub_Tolerance(void);
	void Show_Tolerance_Help(void);
	
private:
	
	QWidget								*d_results_widget;
	QLineEdit 							*d_tolerance_edit;
	QLineEdit 							*d_measured_range_edit;
	QLineEdit							*d_calculated_tolerance_edit;
	QLineEdit							*d_calculated_sub_tolerance_edit;
	QToolButton							*d_apply_button;
	QToolButton							*d_help_button;
	QSlider								*d_sub_tolerance_slider;
	TToleranceHelpDialog				*d_tolerance_help_dialog;

	TToleranceType						d_tolerance_type;
	double								d_tolerance_a;
	double								d_tolerance_b;
	double								d_tolerance_c;
	double								d_tolerance_d;
	double								d_nominal_length;

	bool Analyze_Tolerance(const QString &tolerance_text);
	void Update_Display(void);

// NOT IMPLEMENTED
	TToleranceWidget(const TToleranceWidget&);
	TToleranceWidget& operator=(const TToleranceWidget&);
};

#endif
